@@ -400,7 +400,7 @@ unsigned int TXT_Font[][11]= {
    0x0000,  // ************ ****
   },
 
-  { // 0x40 =  = NC
+  { // 0x40 = § = NC
    0x0000,  // ************ ****
    0x1F80,  // ***######*** ****
    0x30C0,  // **##****##** ****
@@ -725,7 +725,7 @@ unsigned int TXT_Font[][11]= {
    0x0000,  // ************ ****
    0x0000   // ************ ****
   },
-  { // 0x5B =  = NC
+  { // 0x5B = Ä = NC
    0x18C0,  // ***##***##** ****
    0x0000,  // ************ ****
    0x1FC0,  // ***#######** ****
@@ -737,7 +737,7 @@ unsigned int TXT_Font[][11]= {
    0x0000,  // ************ ****
    0x0000   // ************ ****
   },
-  { // 0x5C =  = NC
+  { // 0x5C = Ö = NC
    0x18C0,  // ***##***##** ****
    0x0000,  // ************ ****
    0x1FC0,  // ***#######** ****
@@ -749,7 +749,7 @@ unsigned int TXT_Font[][11]= {
    0x0000,  // ************ ****
    0x0000   // ************ ****
   },
-  { // 0x5D =  = NC
+  { // 0x5D = Ü = NC
    0x18C0,  // ***##***##** ****
    0x0000,  // ************ ****
    0x3060,  // **##*****##* ****
@@ -786,7 +786,7 @@ unsigned int TXT_Font[][11]= {
    0x7FE0   // *##########* ****
   },
 
-  { // 0x60 =  = NC
+  { // 0x60 = ° = NC
    0x0000,  // ************ ****
    0x0000,  // *****####*** ****
    0x0000,  // ****##**##** ****
@@ -1111,7 +1111,7 @@ unsigned int TXT_Font[][11]= {
    0x0000,  // ************ ****
    0x0000   // ************ ****
   },
-  { // 0x7B =  = NC
+  { // 0x7B = ä = NC
    0x0000,  // ************ ****
    0x3180,  // **##***##*** ****
    0x0000,  // ************ ****
@@ -1123,7 +1123,7 @@ unsigned int TXT_Font[][11]= {
    0x0000,  // ************ ****
    0x0000   // ************ ****
   },
-  { // 0x7C =  = NC
+  { // 0x7C = ö = NC
    0x0000,  // ************ ****
    0x18C0,  // ***##***##** ****
    0x0000,  // ************ ****
@@ -1135,7 +1135,7 @@ unsigned int TXT_Font[][11]= {
    0x0000,  // ************ ****
    0x0000   // ************ ****
   },
-  { // 0x7D =  = NC
+  { // 0x7D = ü = NC
    0x0000,  // ************ ****
    0x10C0,  // ***##***##** ****
    0x0000,  // ************ ****
@@ -1147,7 +1147,7 @@ unsigned int TXT_Font[][11]= {
    0x0000,  // ************ ****
    0x0000   // ************ ****
   },
-  { // 0x7E =  = NC
+  { // 0x7E = ß = NC
    0x0000,  // ************ ****
    0x0F80,  // ****#####*** ****
    0x18C0,  // ***##***##** ****
