@@ -100,45 +100,22 @@ int Storage::cleanSubDir(const char *dir) {
    return bytesDeleted;
 }
 
-Storage *Storage::s_self = 0;
-Storage::StorageSystem Storage::system = Storage::StorageSystemPacked;
-
 Storage::Storage() {
-   s_self=this;
    byteCount=0;
-   storageOption=-1;
    failedFreeSpace=false;
 }
 
 Storage::~Storage() {
 }
 
-void Storage::setSystem(StorageSystem s) {
-   system=s;
-}
-
-Storage *Storage::instance() {
-   if (!s_self) {
-      switch (system) {
-      case StorageSystemLegacy:
-         s_self=new LegacyStorage();
-         break;
-      case StorageSystemPacked:
-      default:
-         s_self=new PackedStorage();
-         break;
-      }
-   }
-   return s_self;
-}
-
-void Storage::setMaxStorage(int maxMB) {
-   storageOption=maxMB;
-}
-
-void Storage::init() {
-   cleanUp();
-   initMaxStorage(storageOption);
+Storage *Storage::CreateInstance(StorageSystem system, int storageLimit) {
+  switch (system) {
+  case StorageSystemLegacy:
+     return new LegacyStorage(storageLimit);
+  case StorageSystemPacked:
+  default:
+     return new PackedStorage(storageLimit);
+  }
 }
 
 void Storage::freeSpace() {
@@ -202,10 +179,10 @@ void Storage::prepareDirectory(tChannelID chan) {
 
 #define TELETEXT_PAGESIZE 972
 
-LegacyStorage::LegacyStorage() {
-   maxBytes=0;
+LegacyStorage::LegacyStorage(int maxMB) {
    fsBlockSize=1;
    pageBytes=TELETEXT_PAGESIZE;
+   initMaxStorage(maxMB);
 }
 
 LegacyStorage::~LegacyStorage() {
@@ -335,7 +312,8 @@ ssize_t LegacyStorage::write(const void *ptr, size_t size, StorageHandle stream)
 
 
 
-PackedStorage::PackedStorage() {
+PackedStorage::PackedStorage(int maxMB)
+ : LegacyStorage(maxMB) {
 }
 #define TOC_SIZE 8
 //The file structure is simple:
@@ -457,8 +435,8 @@ StorageHandle PackedStorage::openForReading(PageID page, bool countAsAccess) {
 
 
 
-cTelePage::cTelePage(PageID t_page, uchar t_flags, uchar t_lang,int t_mag)
-  : mag(t_mag), flags(t_flags), lang(t_lang), page(t_page)
+cTelePage::cTelePage(PageID t_page, uchar t_flags, uchar t_lang,int t_mag, Storage *s)
+  : mag(t_mag), flags(t_flags), lang(t_lang), page(t_page), storage(s)
 {
    memset(pagebuf,' ',26*40);
 }
@@ -473,28 +451,31 @@ void cTelePage::SetLine(int line, uchar *myptr)
 
 void cTelePage::save()
 {
-   Storage *s=Storage::instance();
    unsigned char buf;
    StorageHandle fd;
-   if ( (fd=s->openForWriting(page)) ) {
-      s->write("VTXV4",5,fd);
-      buf=0x01; s->write(&buf,1,fd);
-      buf=mag;  s->write(&buf,1,fd);
-      buf=page.page; s->write(&buf,1,fd);
-      buf=flags; s->write(&buf,1,fd);
-      buf=lang; s->write(&buf,1,fd);
-      buf=0x00; s->write(&buf,1,fd);
-      buf=0x00; s->write(&buf,1,fd);
-      s->write(pagebuf,24*40,fd);
-      s->close(fd);
-   }         
+   if ( (fd=storage->openForWriting(page)) ) {
+      storage->write("VTXV4",5,fd);
+      buf=0x01; storage->write(&buf,1,fd);
+      buf=mag;  storage->write(&buf,1,fd);
+      buf=page.page; storage->write(&buf,1,fd);
+      buf=flags; storage->write(&buf,1,fd);
+      buf=lang; storage->write(&buf,1,fd);
+      buf=0x00; storage->write(&buf,1,fd);
+      buf=0x00; storage->write(&buf,1,fd);
+      storage->write(pagebuf,24*40,fd);
+      storage->close(fd);
+   }
 }
 
+bool cTelePage::IsTopTextPage()
+{
+   return (page.page & 0xFF) > 0x99 || (page.page & 0x0F) > 0x9;
+}
 
-cTxtStatus::cTxtStatus(void)
+cTxtStatus::cTxtStatus(bool storeTopText, Storage* storage)
+ :storeTopText(storeTopText), storage(storage)
 {
    receiver = NULL;
-   currentLiveChannel = tChannelID::InvalidID;
 }
 
 cTxtStatus::~cTxtStatus()
@@ -512,26 +493,20 @@ void cTxtStatus::ChannelSwitch(const cDevice *Device, int ChannelNumber)
    cChannel* newLiveChannel = Channels.GetByNumber(ChannelNumber);
    if (newLiveChannel == NULL) return;
 
-   // ignore if channel hasn't changed
-   if (currentLiveChannel == newLiveChannel->GetChannelID()) return;
-
    // ignore non-live-channel-switching
    if (!Device->IsPrimaryDevice() ||
       ChannelNumber != cDevice::CurrentChannel()) return;
 
-   // At this point it seems to be pretty sure to me, that the live
-   // channel was changed to a new channel and OSDTeletext can
+   // live channel was changed
    // now re-attach the receiver to the new live channel
 
-   currentLiveChannel = newLiveChannel->GetChannelID();
-
    delete receiver;
    receiver = NULL;
 
    int TPid = newLiveChannel->Tpid();
 
    if (TPid) {
-      receiver = new cTxtReceiver(TPid, currentLiveChannel);
+      receiver = new cTxtReceiver(TPid, newLiveChannel->GetChannelID(), storeTopText, storage);
       cDevice::ActualDevice()->AttachReceiver(receiver);
    }
 
@@ -539,11 +514,12 @@ void cTxtStatus::ChannelSwitch(const cDevice *Device, int ChannelNumber)
 }
 
 
-cTxtReceiver::cTxtReceiver(int TPid, tChannelID chan)
+cTxtReceiver::cTxtReceiver(int TPid, tChannelID chan, bool storeTopText, Storage* storage)
  : cReceiver(chan, -1, TPid), cThread("osdteletext-receiver"),
-   chan(chan), TxtPage(0), buffer((188+60)*75), running(false)
+   TxtPage(0), storeTopText(storeTopText), buffer((188+60)*75), storage(storage)
 {
-   Storage::instance()->prepareDirectory(chan);
+   storage->prepareDirectory(ChannelID());
+
    // 10 ms timeout on getting TS frames
    buffer.SetTimeouts(0, 10);
 }
@@ -551,12 +527,8 @@ cTxtReceiver::cTxtReceiver(int TPid, tChannelID chan)
 
 cTxtReceiver::~cTxtReceiver()
 {
-   cReceiver::Detach();
-   if (running) {
-      running=false;
-      buffer.Signal();
-      Cancel(2);
-   }
+   Detach();
+   Activate(false);
    buffer.Clear();
    delete TxtPage;
 }
@@ -569,13 +541,11 @@ void cTxtReceiver::Stop()
 void cTxtReceiver::Activate(bool On)
 {
   if (On) {
-     if (!running) {
-        running=true;
+     if (!Running()) {
         Start();
         }
      }
-  else if (running) {
-     running = false;
+  else if (Running()) {
      buffer.Signal();
      Cancel(2);
      }
@@ -583,15 +553,8 @@ void cTxtReceiver::Activate(bool On)
 
 void cTxtReceiver::Receive(uchar *Data, int Length)
 {
-   int len = Length+60;
-
-   if (!buffer.Check(len)) {
-      // Buffer overrun
-      buffer.Signal();
-      return;
-   }
-   cFrame *frame=new cFrame(Data, len);
-   if (frame && !buffer.Put(frame)) {
+   cFrame *frame=new cFrame(Data, Length);
+   if (!buffer.Put(frame)) {
       // Buffer overrun
       delete frame;
       buffer.Signal();
@@ -600,7 +563,7 @@ void cTxtReceiver::Receive(uchar *Data, int Length)
 
 void cTxtReceiver::Action() {
 
-   while (running) {
+   while (Running()) {
       cFrame *frame=buffer.Get();
       if (frame) {
          uchar *Datai=frame->Data();
@@ -619,7 +582,6 @@ void cTxtReceiver::Action() {
    }
 
    buffer.Clear();
-   running=false;
 }
 
 uchar cTxtReceiver::unham16 (uchar *p)
@@ -630,6 +592,17 @@ uchar cTxtReceiver::unham16 (uchar *p)
   return (c1 & 0x0F) | (c2 & 0x0F) *16;
 }
 
+void cTxtReceiver::SaveAndDeleteTxtPage()
+{
+  if (TxtPage) {
+     if (storeTopText || !TxtPage->IsTopTextPage()) {
+        TxtPage->save();
+        delete TxtPage;
+        TxtPage=NULL;
+     }
+  }
+}
+
 void cTxtReceiver::DecodeTXT(uchar* TXT_buf)
 {
    // Format of buffer:
@@ -680,11 +653,7 @@ void cTxtReceiver::DecodeTXT(uchar* TXT_buf)
       // Page no, 10- and 1-digit
 
       if (b1 == 0xff) break;
-      if (TxtPage) {
-         TxtPage->save();
-         delete TxtPage;
-         TxtPage=NULL;
-      }
+      SaveAndDeleteTxtPage();
 
       b2 = unham16 (ptr+2); // Sub-code 0..6 + C4
       b3 = unham16 (ptr+4); // Sub-code 8..13 + C5,C6
@@ -707,7 +676,7 @@ void cTxtReceiver::DecodeTXT(uchar* TXT_buf)
       pgno = mag8 * 256 + b1;
       subno = (b2 + b3 * 256) & 0x3f7f;         // Sub Page Number
 
-      TxtPage = new cTelePage(PageID(chan, pgno, subno), flags, lang, mag);
+      TxtPage = new cTelePage(PageID(ChannelID(), pgno, subno), flags, lang, mag, storage);
       TxtPage->SetLine((int)line,(uchar *)ptr);
       break;
       }
@@ -718,11 +687,7 @@ void cTxtReceiver::DecodeTXT(uchar* TXT_buf)
       }
    /*case 23: 
       {
-      if (TxtPage) {
-         TxtPage->save();
-         delete TxtPage;
-         TxtPage=NULL;
-      }
+      SaveAndDeleteTxtPage();
       break;
       }*/
    default:
